@@ -1,18 +1,18 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
 from datetime import timedelta
 
+
 from django import forms
 from django.db.models import Q
 from django.db.models.sql.constants import QUERY_TERMS
+from django.utils import six
+from django.utils.timezone import now
 from django.utils.translation import ugettext_lazy as _
 
-# timezone support is new in Django 1.4.
-try:
-    from django.utils.timezone import now
-except ImportError:
-    from datetime import datetime
-    now = datetime.now
+from .fields import RangeField, LookupTypeField
 
-from django_filters.fields import RangeField, LookupTypeField
 
 __all__ = [
     'Filter', 'CharFilter', 'BooleanFilter', 'ChoiceFilter',
@@ -21,6 +21,7 @@ __all__ = [
     'RangeFilter', 'DateRangeFilter', 'AllValuesFilter',
 ]
 
+
 LOOKUP_TYPES = sorted(QUERY_TERMS)
 
 
@@ -45,11 +46,13 @@ class Filter(object):
     @property
     def field(self):
         if not hasattr(self, '_field'):
-            if self.lookup_type is None or isinstance(self.lookup_type, (list, tuple)):
+            if (self.lookup_type is None or
+                    isinstance(self.lookup_type, (list, tuple))):
                 if self.lookup_type is None:
                     lookup = [(x, x) for x in LOOKUP_TYPES]
                 else:
-                    lookup = [(x, x) for x in LOOKUP_TYPES if x in self.lookup_type]
+                    lookup = [
+                        (x, x) for x in LOOKUP_TYPES if x in self.lookup_type]
                 self._field = LookupTypeField(self.field_class(
                     required=self.required, widget=self.widget, **self.extra),
                     lookup, required=self.required, label=self.label)
@@ -62,9 +65,9 @@ class Filter(object):
         if not value:
             return qs
         if isinstance(value, (list, tuple)):
-            lookup = str(value[1])
+            lookup = six.text_type(value[1])
             if not lookup:
-                lookup = 'exact'  # we fallback to exact if no choice for lookup is provided
+                lookup = 'exact'  # fallback to exact if lookup is not provided
             value = value[0]
         else:
             lookup = self.lookup_type
@@ -98,9 +101,7 @@ class MultipleChoiceFilter(Filter):
 
     def filter(self, qs, value):
         value = value or ()
-        # TODO: this is a bit of a hack, but ModelChoiceIterator doesn't have a
-        # __len__ method
-        if len(value) == len(list(self.field.choices)):
+        if len(value) == len(self.field.choices):
             return qs
         q = Q()
         for v in value:
@@ -137,7 +138,8 @@ class RangeFilter(Filter):
 
     def filter(self, qs, value):
         if value:
-            return qs.filter(**{'%s__range' % self.name: (value.start, value.stop)})
+            lookup = '%s__range' % self.name
+            return qs.filter(**{lookup: (value.start, value.stop)})
         return qs
 
 
@@ -166,7 +168,8 @@ class DateRangeFilter(ChoiceFilter):
     }
 
     def __init__(self, *args, **kwargs):
-        kwargs['choices'] = [(key, value[0]) for key, value in self.options.iteritems()]
+        kwargs['choices'] = [
+            (key, value[0]) for key, value in six.iteritems(self.options)]
         super(DateRangeFilter, self).__init__(*args, **kwargs)
 
     def filter(self, qs, value):
@@ -180,6 +183,7 @@ class DateRangeFilter(ChoiceFilter):
 class AllValuesFilter(ChoiceFilter):
     @property
     def field(self):
-        qs = self.model._default_manager.distinct().order_by(self.name).values_list(self.name, flat=True)
+        qs = self.model._default_manager.distinct()
+        qs = qs.order_by(self.name).values_list(self.name, flat=True)
         self.extra['choices'] = [(o, o) for o in qs]
         return super(AllValuesFilter, self).field
