@@ -1,27 +1,33 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
 from copy import deepcopy
 
 from django import forms
 from django.db import models
 from django.db.models.fields import FieldDoesNotExist
 from django.db.models.related import RelatedObject
+from django.utils import six
 from django.utils.datastructures import SortedDict
 from django.utils.text import capfirst
 
 try:
     from django.db.models.constants import LOOKUP_SEP
-except ImportError:  # Django < 1.5 fallback
-    from django.db.models.sql.constants import LOOKUP_SEP
+except ImportError:  # pragma: nocover
+    # Django < 1.5 fallback
+    from django.db.models.sql.constants import LOOKUP_SEP  # noqa
+
+from .filters import (Filter, CharFilter, BooleanFilter,
+    ChoiceFilter, DateFilter, DateTimeFilter, TimeFilter, ModelChoiceFilter,
+    ModelMultipleChoiceFilter, NumberFilter)
 
-from django_filters.filters import Filter, CharFilter, BooleanFilter, \
-    ChoiceFilter, DateFilter, DateTimeFilter, TimeFilter, ModelChoiceFilter, \
-    ModelMultipleChoiceFilter, NumberFilter
 
 ORDER_BY_FIELD = 'o'
 
 
 def get_declared_filters(bases, attrs, with_base_filters=True):
     filters = []
-    for filter_name, obj in attrs.items():
+    for filter_name, obj in list(attrs.items()):
         if isinstance(obj, Filter):
             obj = attrs.pop(filter_name)
             if getattr(obj, 'name', None) is None:
@@ -32,11 +38,11 @@ def get_declared_filters(bases, attrs, with_base_filters=True):
     if with_base_filters:
         for base in bases[::-1]:
             if hasattr(base, 'base_filters'):
-                filters = base.base_filters.items() + filters
+                filters = list(base.base_filters.items()) + filters
     else:
         for base in bases[::-1]:
             if hasattr(base, 'declared_filters'):
-                filters = base.declared_filters.items() + filters
+                filters = list(base.declared_filters.items()) + filters
 
     return SortedDict(filters)
 
@@ -59,12 +65,11 @@ def get_model_field(model, f):
         rel, model, direct, m2m = opts.get_field_by_name(parts[-1])
     except FieldDoesNotExist:
         return None
-    if not direct:
-        return rel.field.rel.to_field
     return rel
 
 
-def filters_for_model(model, fields=None, exclude=None, filter_for_field=None):
+def filters_for_model(model, fields=None, exclude=None, filter_for_field=None,
+                      filter_for_reverse_field=None):
     field_dict = SortedDict()
     opts = model._meta
     if fields is None:
@@ -76,7 +81,10 @@ def filters_for_model(model, fields=None, exclude=None, filter_for_field=None):
         if field is None:
             field_dict[f] = None
             continue
-        filter_ = filter_for_field(field, f)
+        if isinstance(field, RelatedObject):
+            filter_ = filter_for_reverse_field(field, f)
+        else:
+            filter_ = filter_for_field(field, f)
         if filter_:
             field_dict[f] = filter_
     return field_dict
@@ -101,14 +109,18 @@ class FilterSetMetaclass(type):
             # We are defining FilterSet itself here
             parents = None
         declared_filters = get_declared_filters(bases, attrs, False)
-        new_class = super(FilterSetMetaclass, cls).__new__(cls, name, bases, attrs)
+        new_class = super(
+            FilterSetMetaclass, cls).__new__(cls, name, bases, attrs)
 
         if not parents:
             return new_class
 
-        opts = new_class._meta = FilterSetOptions(getattr(new_class, 'Meta', None))
+        opts = new_class._meta = FilterSetOptions(
+            getattr(new_class, 'Meta', None))
         if opts.model:
-            filters = filters_for_model(opts.model, opts.fields, opts.exclude, new_class.filter_for_field)
+            filters = filters_for_model(opts.model, opts.fields, opts.exclude,
+                                        new_class.filter_for_field,
+                                        new_class.filter_for_reverse_field)
             filters.update(declared_filters)
         else:
             filters = declared_filters
@@ -144,21 +156,24 @@ FILTER_FOR_DBFIELD_DEFAULTS = {
     models.OneToOneField: {
         'filter_class': ModelChoiceFilter,
         'extra': lambda f: {
-            'queryset': f.rel.to._default_manager.complex_filter(f.rel.limit_choices_to),
+            'queryset': f.rel.to._default_manager.complex_filter(
+                f.rel.limit_choices_to),
             'to_field_name': f.rel.field_name,
         }
     },
     models.ForeignKey: {
         'filter_class': ModelChoiceFilter,
         'extra': lambda f: {
-            'queryset': f.rel.to._default_manager.complex_filter(f.rel.limit_choices_to),
+            'queryset': f.rel.to._default_manager.complex_filter(
+                f.rel.limit_choices_to),
             'to_field_name': f.rel.field_name
         }
     },
     models.ManyToManyField: {
         'filter_class': ModelMultipleChoiceFilter,
         'extra': lambda f: {
-            'queryset': f.rel.to._default_manager.complex_filter(f.rel.limit_choices_to),
+            'queryset': f.rel.to._default_manager.complex_filter(
+                f.rel.limit_choices_to),
         }
     },
     models.DecimalField: {
@@ -201,10 +216,6 @@ FILTER_FOR_DBFIELD_DEFAULTS = {
         'filter_class': CharFilter,
     },
 }
-if hasattr(models, "XMLField"):
-    FILTER_FOR_DBFIELD_DEFAULTS[models.XMLField] = {
-        'filter_class': CharFilter,
-    }
 
 
 class BaseFilterSet(object):
@@ -238,19 +249,22 @@ class BaseFilterSet(object):
     def qs(self):
         if not hasattr(self, '_qs'):
             qs = self.queryset.all()
-            for name, filter_ in self.filters.iteritems():
+            for name, filter_ in six.iteritems(self.filters):
                 try:
                     if self.is_bound:
                         data = self.form[name].data
                     else:
-                        data = self.form.initial.get(name, self.form[name].field.initial)
+                        data = self.form.initial.get(
+                            name, self.form[name].field.initial)
                     val = self.form.fields[name].clean(data)
                     qs = filter_.filter(qs, val)
                 except forms.ValidationError:
                     pass
             if self._meta.order_by:
                 try:
-                    value = self.form.fields[self.order_by_field].clean(self.form[self.order_by_field].data)
+                    order_field = self.form.fields[self.order_by_field]
+                    data = self.form[self.order_by_field].data
+                    value = order_field.clean(data)
                     if value:
                         qs = qs.order_by(value)
                 except forms.ValidationError:
@@ -258,12 +272,18 @@ class BaseFilterSet(object):
             self._qs = qs
         return self._qs
 
+    def count(self):
+        return self.qs.count()
+
     @property
     def form(self):
         if not hasattr(self, '_form'):
-            fields = SortedDict([(name, filter_.field) for name, filter_ in self.filters.iteritems()])
+            fields = SortedDict([
+                (name, filter_.field)
+                for name, filter_ in six.iteritems(self.filters)])
             fields[self.order_by_field] = self.ordering_field
-            Form = type('%sForm' % self.__class__.__name__, (self._meta.form,), fields)
+            Form = type(str('%sForm' % self.__class__.__name__),
+                        (self._meta.form,), fields)
             if self.is_bound:
                 self._form = Form(self.data, prefix=self.form_prefix)
             else:
@@ -273,10 +293,17 @@ class BaseFilterSet(object):
     def get_ordering_field(self):
         if self._meta.order_by:
             if isinstance(self._meta.order_by, (list, tuple)):
-                choices = [(f, capfirst(f)) for f in self._meta.order_by]
+                if isinstance(self._meta.order_by[0], (list, tuple)):
+                    # e.g. (('field', 'Display name'), ...)
+                    choices = [(f[0], f[1]) for f in self._meta.order_by]
+                else:
+                    choices = [(f, capfirst(f)) for f in self._meta.order_by]
             else:
-                choices = [(f, capfirst(f)) for f in self.filters]
-            return forms.ChoiceField(label="Ordering", required=False, choices=choices)
+                # use the filter's label if provided
+                choices = [(fltr.name or f, fltr.label or capfirst(f))
+                           for f, fltr in self.filters.items()]
+            return forms.ChoiceField(label="Ordering", required=False,
+                                     choices=choices)
 
     @property
     def ordering_field(self):
@@ -286,7 +313,8 @@ class BaseFilterSet(object):
 
     @classmethod
     def filter_for_field(cls, f, name):
-        filter_for_field = dict(FILTER_FOR_DBFIELD_DEFAULTS, **cls.filter_overrides)
+        filter_for_field = dict(FILTER_FOR_DBFIELD_DEFAULTS)
+        filter_for_field.update(cls.filter_overrides)
 
         default = {
             'name': name,
@@ -299,12 +327,43 @@ class BaseFilterSet(object):
 
         data = filter_for_field.get(f.__class__)
         if data is None:
-            return
+            # could be a derived field, inspect parents
+            for class_ in f.__class__.mro():
+                # skip if class_ is models.Field or object
+                # 1st item in mro() is original class
+                if class_ in (f.__class__, models.Field, object):
+                    continue
+                data = filter_for_field.get(class_)
+                if data:
+                    break
+            if data is None:
+                return
         filter_class = data.get('filter_class')
         default.update(data.get('extra', lambda f: {})(f))
         if filter_class is not None:
             return filter_class(**default)
 
+    @classmethod
+    def filter_for_reverse_field(cls, f, name):
+        rel = f.field.rel
+        queryset = f.model._default_manager.all()
+        default = {
+            'name': name,
+            'label': capfirst(rel.related_name),
+            'queryset': queryset,
+        }
+        if rel.multiple:
+            return ModelMultipleChoiceFilter(**default)
+        else:
+            return ModelChoiceFilter(**default)
 
-class FilterSet(BaseFilterSet):
-    __metaclass__ = FilterSetMetaclass
+
+class FilterSet(six.with_metaclass(FilterSetMetaclass, BaseFilterSet)):
+    pass
+
+
+def filterset_factory(model):
+    meta = type(str('Meta'), (object,), {'model': model})
+    filterset = type(str('%sFilterSet' % model._meta.object_name),
+                     (FilterSet,), {'Meta': meta})
+    return filterset
