@@ -1,10 +1,19 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
 from itertools import chain
-from urllib import urlencode
+try:
+    from urllib.parse import urlencode
+except:
+    from urllib import urlencode  # noqa
 
 from django import forms
 from django.db.models.fields import BLANK_CHOICE_DASH
 from django.forms.widgets import flatatt
-from django.utils.encoding import force_unicode
+try:
+    from django.utils.encoding import force_text
+except:  # pragma: nocover
+    from django.utils.encoding import force_unicode as force_text  # noqa
 from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext as _
 
@@ -26,26 +35,30 @@ class LinkWidget(forms.Widget):
         if value is None:
             value = ''
         final_attrs = self.build_attrs(attrs)
-        output = [u'<ul%s>' % flatatt(final_attrs)]
+        output = ['<ul%s>' % flatatt(final_attrs)]
         options = self.render_options(choices, [value], name)
         if options:
             output.append(options)
         output.append('</ul>')
-        return mark_safe(u'\n'.join(output))
+        return mark_safe('\n'.join(output))
 
     def render_options(self, choices, selected_choices, name):
-        selected_choices = set(force_unicode(v) for v in selected_choices)
+        selected_choices = set(force_text(v) for v in selected_choices)
         output = []
         for option_value, option_label in chain(self.choices, choices):
             if isinstance(option_label, (list, tuple)):
                 for option in option_label:
-                    output.append(self.render_option(name, selected_choices, *option))
+                    output.append(
+                        self.render_option(name, selected_choices, *option))
             else:
-                output.append(self.render_option(name, selected_choices, option_value, option_label))
-        return u'\n'.join(output)
+                output.append(
+                    self.render_option(name, selected_choices,
+                                       option_value, option_label))
+        return '\n'.join(output)
 
-    def render_option(self, name, selected_choices, option_value, option_label):
-        option_value = force_unicode(option_value)
+    def render_option(self, name, selected_choices,
+                      option_value, option_label):
+        option_value = force_text(option_value)
         if option_label == BLANK_CHOICE_DASH[0][1]:
             option_label = _("All")
         data = self.data.copy()
@@ -58,7 +71,7 @@ class LinkWidget(forms.Widget):
         return self.option_string() % {
              'attrs': selected and ' class="selected"' or '',
              'query_string': url,
-             'label': force_unicode(option_label)
+             'label': force_text(option_label)
         }
 
     def option_string(self):
@@ -76,7 +89,7 @@ class RangeWidget(forms.MultiWidget):
         return [None, None]
 
     def format_output(self, rendered_widgets):
-        return u'-'.join(rendered_widgets)
+        return '-'.join(rendered_widgets)
 
 
 class LookupTypeWidget(forms.MultiWidget):
